@@ -4,10 +4,11 @@ from collections import namedtuple
 from six import string_types
 from weakref import WeakValueDictionary, WeakSet
 
-from .columns import DerivedColumn, Column, DerivedTableColumn, StaticColumn, JoinColumn
+from .columns import DerivedColumn, Column, DerivedTableColumn, StaticColumn, JoinColumn, describe_column
 from .row import TableRow
 from .exceptions import InvalidData
 from .index import Index
+from .aggregation import Aggregation
 
 log = logging.basicConfig()
 
@@ -123,8 +124,8 @@ class Table(object):
         try:
             return [c.description for c in self._all_columns]
         except AttributeError as ae:
-            import pdb
-            pdb.set_trace()
+            log.exception(ae[0])
+            raise RuntimeError(ae[0])
 
     def __iter__(self):
         """Iterate through the rows of this table.
@@ -419,11 +420,124 @@ class Table(object):
             out.append(
                 '| %s |' % (' | '.join(str(c).ljust(l) for l, c in zip(cl, r)))
             )
-        format_row(c.description for c in self._columns)
+        format_row(self._column_descriptions)
         for r in self:
             format_row(r)
         return '\n'.join(out)
 
+    def aggregate(self, keys, aggregations):
+        """Summarize a table by grouping by one or more keys, and then
+        apply aggregation functions to generate additional summarized columns.
+
+        Aggregations are specified as a list of triples in the form:
+        (column name (str), column type (type), column function (callable))
+
+        The column function should be a function that returns the type
+        specified in the 2nd column. It's input will be each of the
+        sub-tables speified by the grouping keys.
+
+        >>> from toytable import table_literal
+        >>> t = table_literal(\"\"\"
+        ... | Attack(str)   | Pokemon(str) | Level Obtained(int) | Attack Type(str) |
+        ... | Thunder Shock | Pikachu      | 1                   | Electric         |
+        ... | Tackle        | Pikachu      | 1                   | Normal           |
+        ... | Tail Whip     | Pikachu      | 1                   | Normal           |
+        ... | Growl         | Pikachu      | 5                   | Normal           |
+        ... | Quick Attack  | Pikachu      | 10                  | Normal           |
+        ... | Thunder Wave  | Pikachu      | 13                  | Electric         |
+        ... | Electro Ball  | Pikachu      | 18                  | Electric         |
+        ... | Charm         | Pikachu      | 0                   | Fairy            |
+        ... | Sweet Kiss    | Pikachu      | 0                   | Fairy            |
+        ... \"\"\")
+        >>>
+        >>> agg = t.aggregate(
+        ...     keys=('Pokemon', 'Attack Type'),
+        ...     aggregations = [
+        ...         ('Count', int, lambda t:len(t))
+        ...     ]
+        ... )
+        >>>
+        >>> print agg
+        | Pokemon (str) | Attack Type (str) | Count (int) |
+        | Pikachu       | Normal            | 4           |
+        | Pikachu       | Electric          | 3           |
+        | Pikachu       | Fairy             | 2           |
+
+        :param keys: List of column names to group by
+        :type keys: List of strings
+        :param aggregations: List of aggregations to calculate
+        :type aggregations: list of tuples
+        """
+        i = self.add_index(keys).reindex()
+        return AggregationTable(
+            self,
+            i,
+            keys=keys,
+            aggregations=aggregations
+        )
+
+
+class AggregationTable(Table):
+
+    def __init__(self, table, index, keys, aggregations):
+        self.table = table
+        self.i = index
+        self.keys = keys
+        self.aggregations = [Aggregation(*a) for a in aggregations]
+
+    def _indices_func(self):
+        return xrange(len(self.i.unique_values()))
+
+    def _iter_subtables(self):
+        """Generator function that gives a sequnce of (values, table) which represents
+        this table if it were split by the unique values in the selected columns.
+        """
+        for uv in self.i.unique_values():
+            iterfn = self.i._get_iterator_fn_for_value(uv)
+            yield uv, DerivedTable(
+                indices_func=iterfn,
+                columns=self.table._columns
+            )
+
+    @property
+    def column_names(self):
+        return list(self.keys) + [a.name for a in self.aggregations]
+
+    @property
+    def column_types(self):
+        return (
+            [self.table._get_column(cn).type for cn in self.keys] +
+            [a.type for a in self.aggregations]
+        )
+
+    def get_row(self, row):
+        row_keys, subtable = itertools.islice(
+            self._iter_subtables(),
+            row,
+            row + 1
+        ).next()
+        r = row_keys + tuple(a(subtable) for a in self.aggregations)
+        return TableRow(r, self.column_names)
+
+    def __iter__(self):
+        cn = self.column_names
+        for row_keys, subtable in self._iter_subtables():
+            r = row_keys + tuple(a(subtable) for a in self.aggregations)
+            yield TableRow(r, self.column_names)
+
+    @property
+    def schema(self):
+        return zip(self.column_names, self.column_types)
+
+    def __getattr__(self, key):
+        return ('Pikachu', 'Normal', 4)
+
+    @property
+    def _column_descriptions(self):
+        return (
+            [describe_column(name, typ) for (name, typ) in self.schema]
+        )
+
 
 class DerivedTable(Table):
 
@@ -604,3 +718,4 @@ class JoinTable(DerivedTable):
                     (jc._column[ji] for jc in jcs)
                 )
             yield TableRow(r, s)
+ow(r, s)
